@@ -29,23 +29,6 @@ struct var_length_char_buffer {
 };
 typedef struct var_length_char_buffer vlc_buf_t;
 
-// template <typename T>
-// struct var_length_vec_buffer_initial {
-//   std::vector<uint64_t> offsets;  // vector for offset values
-//   std::vector<T> data;            // vector for data values
-// };
-// using vli_buf_t_old = struct var_length_vec_buffer_initial<int32_t>;
-// using vld_buf_t_old = struct var_length_vec_buffer_initial<double>;
-
-// THIS WORKS
-// template <typename T>
-// struct var_length_vec_buffer {
-// public:
-//   std::vector<uint64_t> offsets;  // vector for offset values
-//   std::vector<T> data;            // vector for data values
-// };
-// using vli_buf_t = struct var_length_vec_buffer<int32_t>;
-// using vld_buf_t = struct var_length_vec_buffer<double>;
 
 struct var_length_vec_buffer {
 public:
@@ -63,8 +46,9 @@ struct query_buffer {
     //void *ptr;                    	// pointer to data as an alternative
     std::vector<int8_t> vec;        	// vector of int8_t as a memory container
     tiledb_datatype_t dtype;        	// data type
-    R_xlen_t ncells;                	// extent
-    size_t size;                    	// element size
+    R_xlen_t ncells;                	// extent, i.e. number elements
+    size_t size;                    	// element size, i.e. sizeof() given data type
+    int32_t numvar;                     // number of elements per cells, generally fixed
     std::vector<uint8_t> validity_map;  // for nullable vectors
     bool nullable;                      // flag
 };
