@@ -300,6 +300,12 @@ selected_ranges(x) <- list(cbind(2,2), cbind(2,2))
 val <- x[]
 expect_equal(nrow(val), 0L)
 
+selected_ranges(x) <- list()            # unset
+selected_points(x) <- list(2,2)         # same, but via points
+val <- x[]
+expect_equal(nrow(val), 0L)
+
+
 unlink(tmp, recursive = TRUE)
 #})
 
@@ -329,6 +335,11 @@ selected_ranges(x) <- list(cbind(1,21), cbind(10,100))
 val <- x[]
 expect_equal(nrow(val), 1L)
 
+selected_ranges(x) <- list()
+selected_points(x) <- list(21, 100)
+val <- x[]
+expect_equal(nrow(val), 0L)
+
 unlink(tmp, recursive = TRUE)
 #})
 
@@ -361,6 +372,10 @@ matlist <- list(cbind(1,4), cbind(1,4), NULL, cbind(1,2))
 selected_ranges(A) <- matlist
 expect_equal(nrow(A[]), 2L)
 
+selected_ranges(A) <- list()
+selected_points(A) <- list(1, 2, NULL, 4)
+expect_equal(nrow(A[]), 0L)
+
 unlink(tmp, recursive = TRUE)
 #})
 
@@ -599,7 +614,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
@@ -630,7 +645,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
@@ -661,7 +676,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
@@ -692,7 +707,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
@@ -723,7 +738,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
@@ -754,7 +769,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
@@ -785,7 +800,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[as.integer64(rep(1:4,each=4)), as.integer64(rep(1:4,4))] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[as.integer64(1:2), as.integer64(2:3)]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], as.integer64(c(1L, 1L, 2L, 2L)))
@@ -817,7 +832,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[as.integer64(rep(1:4,each=4)), as.integer64(rep(1:4,4))] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[as.integer64(1:2), as.integer64(2:3)]
   expect_equal(newdata[,"a"], c(2L, 3L, 6L, 7L))
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
@@ -868,7 +883,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   #A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
   expect_equal(newdata[,"cols"], c(2L, 3L, 2L, 3L))
@@ -924,7 +939,7 @@ if (requireNamespace("bit64", quietly=TRUE)) {
   ## or with indices
   A[rep(1:4,each=4), rep(1:4,4)] <- data
 
-  A <- tiledb_array(uri = tmp, as.data.frame=TRUE)
+  A <- tiledb_array(uri = tmp, as.data.frame=TRUE, query_layout="ROW_MAJOR")
   newdata <- A[1:2, 2:3]
   expect_equal(newdata[,"rows"], c(1L, 1L, 2L, 2L))
   expect_equal(newdata[,"cols"], c(2L, 3L, 2L, 3L))
@@ -1351,7 +1366,7 @@ data <- array(1:64, dim = c(4,4,4))
 A <- tiledb_array(uri = uri)
 A[] <- data
 
-A <- tiledb_array(uri = uri, return_as="data.frame")
+A <- tiledb_array(uri = uri, return_as="data.frame", query_layout="ROW_MAJOR")
 res <- A[2,2,2]
 expect_equal(res[, "a", drop=TRUE], 22)
 res <- A[2,2:3,2]
@@ -1415,28 +1430,32 @@ expect_identical(attrs(arr), character(length = 0L))
 attrs(arr) <- NA_character_
 expect_true(is.na(attrs(arr)))
 
+v <- tiledb_version()
+if (v[["major"]] == 2L && v[["minor"]] %in% c(4L, 10L)) exit_file("Skip remainder for 2.4.* and 2.10.*")
+
 
 ## check for incomplete status on unsuccessful query
 set_allocation_size_preference(256)     # too low for penguins to return something
-array <- tiledb_array(uri, as.data.frame=TRUE)
+array <- tiledb_array(uri, as.data.frame=TRUE, query_layout="ROW_MAJOR")
 expect_warning(res <- array[])          # warning emitted
 expect_equal(attr(res, "query_status"), "INCOMPLETE") # and query status reported
 
-
-## check for batched operation
-set_allocation_size_preference(1024)
-arr <- tiledb_array(uri, as.data.frame=TRUE)
-lst <- tiledb:::createBatched(arr)
-res1 <- tiledb:::fetchBatched(arr, lst)
-expect_false(completedBatched(lst))
-res2 <- tiledb:::fetchBatched(arr, lst)
-expect_false(completedBatched(lst))
-res3 <- tiledb:::fetchBatched(arr, lst)
-expect_false(completedBatched(lst))
-res4 <- tiledb:::fetchBatched(arr, lst)
-expect_true(completedBatched(lst))
-expect_equal(nrow(res1) + nrow(res2) + nrow(res3) + nrow(res4), 344)
-
+if (Sys.getenv("IS_COVR", "no") == "no") {
+    ## check for batched operation -- but not in coverage
+    set_allocation_size_preference(1024)
+    arr <- tiledb_array(uri, as.data.frame=TRUE)
+    lst <- tiledb:::createBatched(arr)
+    res1 <- tiledb:::fetchBatched(arr, lst)
+    expect_false(completedBatched(lst))
+    res2 <- tiledb:::fetchBatched(arr, lst)
+    expect_false(completedBatched(lst))
+    res3 <- tiledb:::fetchBatched(arr, lst)
+    expect_false(completedBatched(lst))
+    res4 <- tiledb:::fetchBatched(arr, lst)
+    expect_true(completedBatched(lst))
+    expect_equal(nrow(res1) + nrow(res2) + nrow(res3) + nrow(res4), 344)
+    set_allocation_size_preference(1024*1024)
+}
 
 ## check NAs in character column
 library(palmerpenguins)
